@@ -14,9 +14,8 @@
 """Deploy notebook to nuclio"""
 import json
 import os
-from os import environ
+import tempfile
 from operator import itemgetter
-from tempfile import mktemp
 from time import sleep, time
 from datetime import datetime
 
@@ -64,20 +63,20 @@ def find_dashboard_url(dashboard_url=''):
 
     def with_prefix(url):
         api_prefix = '/api'
-        if environ.get('NUCLIO_DROP_API'):
+        if os.environ.get('NUCLIO_DROP_API'):
             api_prefix = ''
         return url + api_prefix
 
     if dashboard_url:
         return with_prefix(dashboard_url)
 
-    if 'NUCLIO_DASHBOARD_URL' in environ:
-        return with_prefix(environ.get('NUCLIO_DASHBOARD_URL'))
+    if 'NUCLIO_DASHBOARD_URL' in os.environ:
+        return with_prefix(os.environ.get('NUCLIO_DASHBOARD_URL'))
 
     for service, endpoint in service_names.items():
         env_name = service + '_SERVICE_PORT'
-        if env_name in environ:
-            port = environ.get(env_name) or '8070'
+        if env_name in os.environ:
+            port = os.environ.get(env_name) or '8070'
             return with_prefix('http://{}:{}'.format(endpoint, port))
 
     return with_prefix('http://localhost:8070')
@@ -230,7 +229,7 @@ def deploy_code(code, dashboard_url='', name='', project='', handler='',
     if files:
         zip_path = archive
         if url_target:
-            zip_path = mktemp('.zip')
+            zip_path = tempfile.NamedTemporaryFile(suffix='.zip', delete=False).name
         build_zip(zip_path, newconfig, code, files, lang)
         upload_file(zip_path, archive, True)
         newconfig = get_archive_config(name, archive)
@@ -485,7 +484,18 @@ def get_deploy_status(api_address,
         auth_info = AuthInfo.from_envvar()
     last_time = last_time or (time() * 1000.0)
     address = ''
-    function_status = get_function_status(api_address, name, auth_info=auth_info)
+    resp = get_function(api_address, name, auth_info)
+    if not resp.ok:
+        raise DeployError('error: failed getting function {}'.format(name))
+
+    function_config = resp.json()
+    function_status = function_config.get('status', {})
+    internal_invocation_urls, external_invocation_urls = _resolve_function_addresses(api_address,
+                                                                                     name,
+                                                                                     function_config,
+                                                                                     auth_info)
+    function_status["internalInvocationUrls"] = internal_invocation_urls
+    function_status["externalInvocationUrls"] = external_invocation_urls
 
     http_port = function_status.get('httpPort', 0)
     state, last_time, outputs = process_resp({'status': function_status}, last_time, verbose, log_message=False)
